require 'raven'

module Fluent
  class RavenOutput < BufferedOutput
    Fluent::Plugin.register_output('raven', self)

    config_param :server, :string
    config_param :ssl_verification, :bool, default: false
    config_param :timeout, :integer, default: 1
    config_param :open_timeout, :integer, default: 1
    config_param :raven_log_path, :string, default: nil
    config_param :raven_log_level, :string, default: 'info'

    def configure(conf)
      super

      @base_configuration = Raven::Configuration.new
      @base_configuration.server = conf['server']
      @base_configuration.public_key = 'not_used'
      @base_configuration.secret_key = 'not_used'
      @base_configuration.ssl_verification = conf['ssl_verification']
      @base_configuration.timeout = conf['timeout']
      @base_configuration.open_timeout = conf['open_timeout']

      if conf['raven_log_path'].nil?
        log.warn("`raven_log_level` is meaningless when `raven_log_path` isn't set")
      end

      Raven.configure do |config|
        config.logger = if conf['raven_log_path'].nil?
                          log
                        else
                          Logger.new(conf['raven_log_path'], log_level(conf['raven_log_level']))
                        end
      end
    end

    def multi_workers_ready?
      true
    end

    def start
      super
    end

    def shutdown
      super
    end

    def format(tag, time, record)
      [tag, time, record].to_msgpack
    end

    def write(chunk)
      chunk.msgpack_each do |tag, time, record|
        auth_header = record['auth_header']
        data        = record['data']
        options     = Hash[record['options'].map { |key, value| [key.to_sym, value] }] if record['options']
        project_id  = record['project_id']
        send_to_sentry(auth_header, data, project_id, options)
      end
    end

    private

    def send_to_sentry(auth_header, data, project_id, options)
      config = @base_configuration.dup
      config.project_id = project_id
      transport = ::Raven::Transports::HTTP.new(config)
      transport.send_event(auth_header, data, options)
    end

    LOG_LEVELS = {
      'debug' => Logger::DEBUG,
      'info' => Logger::INFO,
      'warn' => Logger::WARN,
      'error' => Logger::ERROR,
      'fatal' => Logger::FATAL,
      'unknown' => Logger::UNKNOWN,
    }.freeze

    def log_level(str)
      LOG_LEVELS.fetch(str, 'info')
    end
  end
end
